@@ -4,9 +4,9 @@ import warnings
 from dataclasses import dataclass, field
 
 import numpy as np
+from astropy import units as u
 from astropy.nddata import NDData
 from astropy.utils.decorators import deprecated_attribute
-from astropy import units as u
 from specutils import Spectrum1D
 
 from specreduce.core import _ImageParser
@@ -32,9 +32,10 @@ class Background(_ImageParser):
     ----------
     image : `~astropy.nddata.NDData`-like or array-like
         image with 2-D spectral image data
-    traces : List
-        list of trace objects (or integers to define FlatTraces) to
-        extract the background
+    traces : trace, int, float (single or list)
+        Individual or list of trace object(s) (or integers/floats to define
+        FlatTraces) to extract the background. If None, a FlatTrace at the
+        center of the image (according to `disp_axis`) will be used.
     width : float
         width of extraction aperture in pixels
     statistic: string
@@ -82,16 +83,6 @@ class Background(_ImageParser):
         crossdisp_axis : int
             cross-dispersion axis
         """
-        def _to_trace(trace):
-            if not isinstance(trace, Trace):
-                trace = FlatTrace(self.image, trace)
-
-            # TODO: this check can be removed if/when implemented as a check in FlatTrace
-            if isinstance(trace, FlatTrace):
-                if trace.trace_pos < 1:
-                    raise ValueError('trace_object.trace_pos must be >= 1')
-            return trace
-
         self.image = self._parse_image(self.image)
 
         if self.width < 0:
@@ -100,12 +91,10 @@ class Background(_ImageParser):
             self._bkg_array = np.zeros(self.image.shape[self.disp_axis])
             return
 
-        if isinstance(self.traces, Trace):
-            self.traces = [self.traces]
+        self._set_traces()
 
         bkg_wimage = np.zeros_like(self.image.data, dtype=np.float64)
         for trace in self.traces:
-            trace = _to_trace(trace)
             windows_max = trace.trace.data.max() + self.width/2
             windows_min = trace.trace.data.min() - self.width/2
             if windows_max >= self.image.shape[self.crossdisp_axis]:
@@ -133,17 +122,56 @@ class Background(_ImageParser):
 
         self.bkg_wimage = bkg_wimage
 
+        # mask user-highlighted and invalid values (if any) before taking stats
+        or_mask = (np.logical_or(~np.isfinite(self.image.data), self.image.mask)
+                   if self.image.mask is not None
+                   else ~np.isfinite(self.image.data))
+
         if self.statistic == 'average':
-            self._bkg_array = np.average(self.image.data,
-                                         weights=self.bkg_wimage,
-                                         axis=self.crossdisp_axis)
+            image_ma = np.ma.masked_array(self.image.data, mask=or_mask)
+            self._bkg_array = np.ma.average(image_ma,
+                                            weights=self.bkg_wimage,
+                                            axis=self.crossdisp_axis).data
         elif self.statistic == 'median':
-            med_image = self.image.data.copy()
-            med_image[np.where(self.bkg_wimage) == 0] = np.nan
-            self._bkg_array = np.nanmedian(med_image, axis=self.crossdisp_axis)
+            med_mask = np.logical_or(self.bkg_wimage == 0, or_mask)
+            image_ma = np.ma.masked_array(self.image.data, mask=med_mask)
+            self._bkg_array = np.ma.median(image_ma, axis=self.crossdisp_axis).data
         else:
             raise ValueError("statistic must be 'average' or 'median'")
 
+    def _set_traces(self):
+        """Determine `traces` from input. If an integer/float or list if int/float
+           is passed in, use these to construct FlatTrace objects. These values
+           must be positive. If None (which is initialized to an empty list),
+           construct a FlatTrace using the center of image (according to disp.
+           axis). Otherwise, any Trace object or list of Trace objects can be
+           passed in."""
+
+        if self.traces == []:
+            # assume a flat trace at the image center if nothing is passed in.
+            trace_pos = self.image.shape[self.disp_axis] / 2.
+            self.traces = [FlatTrace(self.image, trace_pos)]
+
+        if isinstance(self.traces, Trace):
+            # if just one trace, turn it into iterable.
+            self.traces = [self.traces]
+            return
+
+        # finally, if float/int is passed in convert to FlatTrace(s)
+        if isinstance(self.traces, (float, int)):  # for a single number
+            self.traces = [self.traces]
+
+        if np.all([isinstance(x, (float, int)) for x in self.traces]):
+            self.traces = [FlatTrace(self.image, trace_pos) for trace_pos in self.traces]
+            return
+
+        else:
+            if not np.all([isinstance(x, Trace) for x in self.traces]):
+                raise ValueError('`traces` must be a `Trace` object or list of '
+                                 '`Trace` objects, a number or list of numbers to '
+                                 'define FlatTraces, or None to use a FlatTrace in '
+                                 'the middle of the image.')
+
     @classmethod
     def two_sided(cls, image, trace_object, separation, **kwargs):
         """
@@ -177,7 +205,7 @@ class Background(_ImageParser):
         crossdisp_axis : int
             cross-dispersion axis
         """
-        image = cls._parse_image(cls, image)
+        image = _ImageParser._get_data_from_image(image) if image is not None else cls.image
         kwargs['traces'] = [trace_object-separation, trace_object+separation]
         return cls(image=image, **kwargs)
 
@@ -214,7 +242,7 @@ class Background(_ImageParser):
         crossdisp_axis : int
             cross-dispersion axis
         """
-        image = cls._parse_image(cls, image)
+        image = _ImageParser._get_data_from_image(image) if image is not None else cls.image
         kwargs['traces'] = [trace_object+separation]
         return cls(image=image, **kwargs)
 
@@ -232,7 +260,7 @@ class Background(_ImageParser):
 
         Returns
         -------
-        Spectrum1D object with same shape as ``image``.
+        `~specutils.Spectrum1D` object with same shape as ``image``.
         """
         image = self._parse_image(image)
         return Spectrum1D(np.tile(self._bkg_array,
@@ -261,11 +289,11 @@ class Background(_ImageParser):
         bkg_image = self.bkg_image(image)
 
         try:
-            return bkg_image.collapse(np.sum, axis=self.crossdisp_axis)
+            return bkg_image.collapse(np.nansum, axis=self.crossdisp_axis)
         except u.UnitTypeError:
             # can't collapse with a spectral axis in pixels because
             # SpectralCoord only allows frequency/wavelength equivalent units...
-            ext1d = np.sum(bkg_image.flux, axis=self.crossdisp_axis)
+            ext1d = np.nansum(bkg_image.flux, axis=self.crossdisp_axis)
             return Spectrum1D(ext1d, bkg_image.spectral_axis)
 
     def sub_image(self, image=None):
@@ -280,7 +308,7 @@ class Background(_ImageParser):
 
         Returns
         -------
-        array with same shape as ``image``
+        `~specutils.Spectrum1D` object with same shape as ``image``.
         """
         image = self._parse_image(image)
 
@@ -313,11 +341,11 @@ class Background(_ImageParser):
         sub_image = self.sub_image(image=image)
 
         try:
-            return sub_image.collapse(np.sum, axis=self.crossdisp_axis)
+            return sub_image.collapse(np.nansum, axis=self.crossdisp_axis)
         except u.UnitTypeError:
             # can't collapse with a spectral axis in pixels because
             # SpectralCoord only allows frequency/wavelength equivalent units...
-            ext1d = np.sum(sub_image.flux, axis=self.crossdisp_axis)
+            ext1d = np.nansum(sub_image.flux, axis=self.crossdisp_axis)
             return Spectrum1D(ext1d, spectral_axis=sub_image.spectral_axis)
 
     def __rsub__(self, image):
