@@ -1,34 +1,21 @@
-import pytest
 import numpy as np
-
-import astropy.units as u
-from astropy.nddata import VarianceUncertainty
+import pytest
 from specutils import Spectrum1D
 
 from specreduce.background import Background
 from specreduce.tracing import FlatTrace, ArrayTrace
 
 
-# NOTE: same test image as in test_extract.py
-# Test image is comprised of 30 rows with 10 columns each. Row content
-# is row index itself. This makes it easy to predict what should be the
-# value extracted from a region centered at any arbitrary Y position.
-image = np.ones(shape=(30, 10))
-for j in range(image.shape[0]):
-    image[j, ::] *= j
-image = Spectrum1D(image * u.DN,
-                   uncertainty=VarianceUncertainty(np.ones_like(image)))
-image_um = Spectrum1D(image.flux,
-                      spectral_axis=np.arange(image.data.shape[1]) * u.um,
-                      uncertainty=VarianceUncertainty(np.ones_like(image.data)))
-
-
-def test_background():
+def test_background(mk_test_img_raw, mk_test_spec_no_spectral_axis,
+                    mk_test_spec_with_spectral_axis):
+    img = mk_test_img_raw
+    image = mk_test_spec_no_spectral_axis
+    image_um = mk_test_spec_with_spectral_axis
     #
     # Try combinations of extraction center, and even/odd
     # extraction aperture sizes.
     #
-    trace_pos = 15.0
+    trace_pos = 15
     trace = FlatTrace(image, trace_pos)
     bkg_sep = 5
     bkg_width = 2
@@ -72,12 +59,29 @@ def test_background():
     assert isinstance(bkg_spec, Spectrum1D)
     sub_spec = bg1.sub_spectrum()
     assert isinstance(sub_spec, Spectrum1D)
+
     # test that width==0 results in no background
     bg = Background.two_sided(image, trace, bkg_sep, width=0)
     assert np.all(bg.bkg_image().flux == 0)
 
+    # test that any NaNs in input image (whether in or outside the window) don't
+    # propagate to _bkg_array (which affects bkg_image and sub_image methods) or
+    # the final 1D spectra.
+    img[0, 0] = np.nan  # out of window
+    img[trace_pos, 0] = np.nan  # in window
+    stats = ['average', 'median']
+
+    for st in stats:
+        bg = Background(img, trace-bkg_sep, width=bkg_width, statistic=st)
+        assert np.isnan(bg.image.flux).sum() == 2
+        assert np.isnan(bg._bkg_array).sum() == 0
+        assert np.isnan(bg.bkg_spectrum().flux).sum() == 0
+        assert np.isnan(bg.sub_spectrum().flux).sum() == 0
+
+
+def test_warnings_errors(mk_test_spec_no_spectral_axis):
+    image = mk_test_spec_no_spectral_axis
 
-def test_warnings_errors():
     # image.shape (30, 10)
     with pytest.warns(match="background window extends beyond image boundaries"):
         Background.two_sided(image, 25, 4, width=3)
@@ -105,3 +109,29 @@ def test_warnings_errors():
 
     with pytest.raises(ValueError, match="width must be positive"):
         Background.two_sided(image, 25, 2, width=-1)
+
+
+def test_trace_inputs(mk_test_img_raw):
+
+    image = mk_test_img_raw
+
+    # When `Background` object is created with no Trace object passed in it should
+    # create a FlatTrace in the middle of the image (according to disp. axis)
+    background = Background(image, width=5)
+    assert np.all(background.traces[0].trace.data == image.shape[1] / 2.)
+
+    # FlatTrace(s) should be created if number or list of numbers is passed in for `traces`
+    background = Background(image, 10., width=5)
+    assert isinstance(background.traces[0], FlatTrace)
+    assert background.traces[0].trace_pos == 10.
+
+    traces = [10., 15]
+    background = Background(image, traces, width=5)
+    for i, trace_pos in enumerate(traces):
+        assert background.traces[i].trace_pos == trace_pos
+
+    # make sure error is raised if input for `traces` is invalid
+    match_str = 'objects, a number or list of numbers to define FlatTraces, ' +\
+                'or None to use a FlatTrace in the middle of the image.'
+    with pytest.raises(ValueError, match=match_str):
+        Background(image, 'non_valid_trace_pos')
