@@ -1,17 +1,16 @@
-import pytest
-
 import numpy as np
-
-import astropy.units as u
+import pytest
+from astropy import units as u
 from astropy.utils.exceptions import AstropyUserWarning
 
-from ..calibration_data import (
+from specreduce.calibration_data import (
     AtmosphericExtinction,
     AtmosphericTransmission,
     SUPPORTED_EXTINCTION_MODELS
 )
 
 
+@pytest.mark.remote_data
 def test_supported_models():
     """
     Test loading of supported models
@@ -22,6 +21,7 @@ def test_supported_models():
         assert len(ext.transmission) > 0
 
 
+@pytest.mark.remote_data
 def test_custom_mag_model():
     """
     Test creation of custom model from Quantity arrays
@@ -33,6 +33,7 @@ def test_custom_mag_model():
     assert len(ext.transmission) > 0
 
 
+@pytest.mark.remote_data
 def test_custom_raw_mag_model():
     """
     Test creation of custom model from Quantity arrays
@@ -44,6 +45,7 @@ def test_custom_raw_mag_model():
     assert len(ext.transmission) > 0
 
 
+@pytest.mark.remote_data
 def test_custom_linear_model():
     """
     Test creation of custom model from Quantity arrays
@@ -55,6 +57,7 @@ def test_custom_linear_model():
     assert len(ext.transmission) > 0
 
 
+@pytest.mark.remote_data
 def test_missing_extinction_unit():
     """
     Test creation of custom model from Quantity arrays
@@ -68,6 +71,7 @@ def test_missing_extinction_unit():
     assert len(ext.transmission) > 0
 
 
+@pytest.mark.remote_data
 def test_transmission_model():
     """
     Test creating of default atmospheric transmission model
